<?php

/**
 * Class Payio_Custom_Order class-payio-gateway-request file.
 *
 * @package PayioLtd/Payio
 */

if (!defined('ABSPATH')) {
    exit;
}


/**
 * Payio_Custom_Order
 */
class Payio_Custom_Order
{
    public function create_new_order()
    {
        //get current user
        $user_id = get_current_user_id();
        $customer_data = WC()->customer;

        //create order
        $order = wc_create_order();

        // get logged in user details`
        if ($user_id !== 0) {
            $user_info = get_userdata($user_id);
            $email = $user_info->user_email;
            $firstname = $user_info->user_firstname;
            $lastname = $user_info->user_lastname;
            $company = $user_info->company;
        }
        // get address from cart
        $shipping_address_1 = $customer_data->shipping_address_1;
        $shipping_address_2 = $customer_data->shipping_address_2;
        $shipping_city       = $customer_data->shipping_city;
        $shipping_state      = $customer_data->shipping_state;
        $shipping_postcode   = $customer_data->shipping_postcode;
        $shipping_country    = $customer_data->shipping_country;

        //create full address for new order
        $address = array(
            'first_name' => $firstname ?? '-',
            'last_name' => $lastname ?? '-',
            'company'    => $company ?? '',
            'email'      => $email ?? 'placeholder@email.com',
            'phone'      => '',
            'address_1'  => empty($shipping_address_1) ? '-' : $shipping_address_1,
            'address_2'  => $shipping_address_2 ?? '',
            'city'       => $shipping_city ?? '',
            'state'      => $shipping_state ?? '',
            'postcode'   => empty($shipping_postcode) ? '-' : $shipping_postcode,
            'country'    => $shipping_country ?? ''
        );

        $this->set_customer_details($order, $address);
        $this->create_shipping($order);
        //get cart
        $cart_data = WC()->cart;
        $items = $cart_data->get_cart();

        // add products from cart to order
        array_map(function ($item) use ($order) {
            $order->add_product(wc_get_product($item['product_id']), 1);
        }, $items);

        $order->calculate_totals();
        $order->update_status('pending');

        //call checkout function
        $gateway_secure_url = (new WC_Payio_Gateway)->init_process_payment(($order->ID), true);
        return esc_url_raw($gateway_secure_url) ?? null;
    }

    public function set_customer_details($order, $address)
    {
        $order->set_address($address, 'billing');
        $order->set_address($address, 'shipping');
    }

    public function create_shipping($order)
    {
        $current_session = WC()->session;
        $chosen_shiping_method = $current_session->get('chosen_shipping_methods');

        $shipping_item = new WC_Order_Item_Shipping();
        // Set the array for tax calculations
        $shipping_tax_item = array(
            'country' => $shipping_country ?? 'GB',
            'state' => '',
            'postcode' => '',
            'city' => '',
        );
        // Loop through shipping packages from WC_Session (They can be multiple in some cases)
        foreach (WC()->cart->get_shipping_packages() as $package_id => $package) {
            // Check if a shipping for the current package exist
            if (WC()->session->__isset('shipping_for_package_' . $package_id)) {
                // Loop through shipping rates for the current package
                foreach (WC()->session->get('shipping_for_package_' . $package_id)['rates'] as $shipping_rate_id => $shipping_rate) {

                    //get info of chosen shipping method
                    if ($chosen_shiping_method[0] == $shipping_rate->get_id()) {
                        $shipping_rate_id     = $shipping_rate->get_id();
                        $shipping_label_name  = $shipping_rate->get_label();
                        $shipping_cost        = $shipping_rate->get_cost();
                    }
                }
            }
        }

        $shipping_item->set_method_title($shipping_label_name);
        $shipping_item->set_method_id($shipping_rate_id);
        $shipping_item->set_total($shipping_cost);
        $shipping_item->calculate_taxes($shipping_tax_item);

        //add shipping item to order
        $order->add_item($shipping_item);
    }
}
