<?php

class Payio_Webhooks
{
    /**
     * payment_update_order
     *
     * @return void
     */
    public function payment_update_order()
    {

        $data = json_decode(file_get_contents('php://input'), true);
        $customer_details = $data['customerDetails'];
        $order_id = $data["orderId"];

        if (is_null($order_id)) {
            echo json_encode('Failed: No order ID');
            die();
        }
        $decoded_token = base64_decode(base64_encode($order_id));
        if ($order_id !== $decoded_token) {
            echo json_encode('Failed: Not authorised');
            die();
        }
        $order = wc_get_order($order_id);
        if (empty($order)) {
            echo json_encode('Failed: No order exists with that order number');
            die();
        }

        //if status exists change status
        $status = $data["status"];
        if ($status) {
            if ($status === "PENDING") {
                $order->update_status('on-hold');
                echo json_encode('Payment status updated to on-hold');
                die();
            } else if ($status === "COMPLETED") {
                $order->update_status('processing');
                echo json_encode('Payment status updated to processing');
                die();
            };
            echo json_encode('Failed: Incorrect status provided');
            die();
        };

        $shipping = $data['shipping'];

        //create full address for new order
        $address = array(
            'first_name' => sanitize_text_field($customer_details['firstName']),
            'last_name'  => sanitize_text_field($customer_details['lastName']),
            'email'      => sanitize_email($customer_details['email']),
            'phone'      => wc_sanitize_phone_number($customer_details['phoneNumber']),
            'address_1'  => sanitize_text_field($customer_details['shippingAddress']),
            'address_2'  => sanitize_text_field($customer_details['shippingAddress2']),
            'city'       => sanitize_text_field($customer_details['shippingCity']),
            'state'      => '', // not included in gateway
            'postcode'   => sanitize_text_field($customer_details['shippingPostcode']),
            'country'    => sanitize_text_field($customer_details['country']),
        );

        //update details
        $order->set_address($address, 'billing');
        $order->set_address($address, 'shipping');

        // get new shipping item
        $items    = (array) $order->get_items('shipping');
        $shipping_method_id  = $shipping['id'];
        $shipping_method_name  = $shipping['name'];
        $cost     = $shipping['cost'];

        // Set the array for tax calculations
        $shipping_tax_item = array(
            'country' =>  $customer_details['country'] ?? 'GB',
            'state' => '',
            'postcode' => '',
            'city' => '',
        );

        // Loop through shipping items
        foreach ($items as $item) {
            $item->set_method_title(__($shipping_method_name));
            $item->set_method_id($shipping_method_id);
            $item->set_total($cost);

            $item->calculate_taxes($shipping_tax_item); // Calculate taxes

            if (isset($new_item) && $new_item) {
                $order->add_item($item);
            }
        }

        $order->calculate_totals();

        die();
    }
}
